/*
    Project: TCTEMP
    Version: V1.18
    File   : tc.h

    Copyright (c) 2006-2009 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#ifndef __TC_H
#define __TC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "tomcrypt.h"

#define TC_MAX_PASSWORD_LEN    64
#define TC_MAX_PASSWORD        (TC_MAX_PASSWORD_LEN + 1)

#define TC_HEADER_ID           0x54525545
#define TC_HEADER_VERSION      0x0004
#define TC_REQ_PROGRAM_VERSION 0x0600

#pragma pack(push, 1)

typedef unsigned __int8  TC_UINT8;
typedef unsigned __int16 TC_UINT16;
typedef unsigned __int32 TC_UINT32;
typedef unsigned __int64 TC_UINT64;

typedef struct
{
    /* not encrypted */
    TC_UINT8  rgbSalt[64];

    /* encrypted */
    TC_UINT32 dwId;
    TC_UINT16 wHeaderVersion;
    TC_UINT16 wReqProgramVersion;
    TC_UINT32 dwCRC32;
    TC_UINT64 ullReserved1;
    TC_UINT64 ullReserved2;
    TC_UINT64 ullHiddenVolumeSize;
    TC_UINT64 ullVolumeSize;
    TC_UINT64 ullEncryptedAreaStart;
    TC_UINT64 ullEncryptedAreaSize;
    TC_UINT32 dwFlags;
    TC_UINT8  rgbReserved[124];
    TC_UINT32 dwCRC32_2;
    TC_UINT8  rgbKeys[256];          /* primary keys || secondary keys || unused   */

} TC_HEADER, *PTC_HEADER;

typedef struct
{
    TC_UINT8  rgbKeys[3*32+3*32];
} TC_KEY, *PTC_KEY;

typedef struct
{
    const struct ltc_cipher_descriptor *pdCipher[3];
    symmetric_key                      sk[3];
    symmetric_key                      sk2[3]; /* secondary key schedule */

    /* temporary data */
    TC_UINT8                           rgbT[16];
    TC_UINT64                          ullI;
    TC_UINT64                          ullIHi;
} TCE_CONTEXT, *PTCE_CONTEXT;

#define TC_DEFAULT_ENTROPY_POOL_SIZE 64
typedef struct
{
    TC_UINT8  *pbWritePos;
    TC_UINT8  *pbPoolEnd;
    TC_UINT32 dwCRC;
    TC_UINT8  rgbPool[TC_DEFAULT_ENTROPY_POOL_SIZE];
} TC_ENTROPY_POOL, *PTC_ENTROPY_POOL;

typedef struct
{
    unsigned cbPasswordSize;
    char     rgbPassword[TC_MAX_PASSWORD];
} TC_PASSWORD, *PTC_PASSWORD;

typedef struct
{
    TC_UINT8    rgbFrequency[256];
    TC_PASSWORD pwTemp;
} TC_PASSWORD_INFO, *PTC_PASSWORD_INFO;

typedef struct
{
    union {
        PKCS5_2_BUFFER   pkcs5_2_state;
        TCE_CONTEXT      ctx;
        TC_PASSWORD_INFO pi;
    };
    TC_KEY      key;
} TC_BUFFER, *PTC_BUFFER;

#pragma pack(pop)

#define TC_AES                 1
#define TC_SERPENT             4
#define TC_TWOFISH             6
#define TC_AES_TWOFISH         7
#define TC_AES_TWOFISH_SERPENT 8
#define TC_SERPENT_AES         9
#define TC_SERPENT_TWOFISH_AES 10
#define TC_TWOFISH_SERPENT     11

#define TC_RIPEMD160           1
#define TC_WHIRLPOOL           3
#define TC_SHA512              4

#define TC_RIPEMD160_ITERATION_COUNT 2000
#define TC_WHIRLPOOL_ITERATION_COUNT 1000
#define TC_SHA512_ITERATION_COUNT    1000

#ifndef TRUE
  typedef int BOOL;
# define TRUE  (1==1)
# define FALSE (1!=1)
#endif

#define TC_GetEntropyPoolSize(pep) ((pep)->pbPoolEnd - (pep)->rgbPool)
void __stdcall TC_InitEntropyPool(PTC_ENTROPY_POOL pep, size_t cbPoolSize);
void __stdcall TC_FillEntropyPool(PTC_ENTROPY_POOL pep, const void *pData,
  size_t cbDataSize);

BOOL __stdcall TC_CipherIsValid(unsigned uCipher);
unsigned __stdcall TC_GetCipherIndex(const char *paszCipherName);
const char * __stdcall TC_GetCipherName(unsigned uCipher);

BOOL __stdcall TC_HashIsValid(unsigned uHash);
unsigned __stdcall TC_GetHashIndex(const char *paszHashName);
const char * __stdcall TC_GetHashName(unsigned uHash);
unsigned __stdcall TC_GetDefaultIterationCount(unsigned uHash);

BOOL __stdcall TC_KeyIsWeak(PTC_KEY pKey, unsigned uCipher);
BOOL __stdcall TC_PasswordIsWeak(PTC_PASSWORD pPassword,
  unsigned uDesiredStrength, PTC_PASSWORD_INFO);

BOOL __stdcall TCE_Init(PTCE_CONTEXT pCtx, PTC_KEY pKey, unsigned uCipher);
void __stdcall TCE_SetIndex(PTCE_CONTEXT pCtx, unsigned __int64 ullILo, unsigned __int64 ullIHi);
void __stdcall TCE_SetDataUnitNumber(PTCE_CONTEXT pCtx,
  unsigned __int64 ullDataUnitNumber);
BOOL __stdcall TCE_Encrypt(PTCE_CONTEXT pCtx, void *pDest, size_t cbDestSize);
BOOL __stdcall TCE_Decrypt(PTCE_CONTEXT pCtx, void *pDest, size_t cbDestSize);
void __stdcall TCE_Close(PTCE_CONTEXT pCtx);

BOOL __stdcall TC_DeriveKey(void *pKey, size_t cbKeySize,
  const void *pPassword, size_t cbPasswordSize,
  const void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  PPKCS5_2_BUFFER pBuffer);
BOOL __stdcall TC_DeriveNonWeakKey(PTC_KEY pKey,
  const void *pPassword, size_t cbPasswordSize,
  void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  unsigned uCipher,
  PPKCS5_2_BUFFER pBuffer);
BOOL __stdcall TC_EncryptHeader(PTC_HEADER pHeader, const void *pPassword,
  size_t cbPasswordSize, unsigned uHash, unsigned uCipher, PTC_BUFFER pBuffer,
  TC_UINT64 ullVolumeSize);
BOOL TC_MakeRandomHeader(PTC_HEADER pHeader, PTC_PASSWORD pPassword,
  PTC_ENTROPY_POOL pEntropyPool, unsigned uHash, unsigned uCipher,
  unsigned uIterationCount, PTC_BUFFER pBuffer, TC_UINT64 ullVolumeSize);

#ifdef __cplusplus
}
#endif

#endif
